package de.uni_passau.fim.PADAS.group3.DataDash.Dataset;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.*;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.config.EnableSpringDataWebSupport;
import org.springframework.data.web.config.EnableSpringDataWebSupport.PageSerializationMode;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.data.domain.Sort;

import java.util.UUID;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

@EnableSpringDataWebSupport(pageSerializationMode = PageSerializationMode.VIA_DTO)
@RestController
@RequestMapping("/api/v1/datasets")
public class DatasetController {
    @Autowired
    private DatasetService datasetService;

    @GetMapping("/id/{id}")
    public ResponseEntity<Dataset> getDatasetById(@PathVariable("id") UUID id) {
        Dataset d = datasetService.getDatasetById(id);
        if (d == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(d, HttpStatus.OK);
    }

    @ResponseStatus(HttpStatus.CREATED)
    @PostMapping
    public Dataset createDataset(@RequestBody Dataset dataset) {
        return datasetService.addDataset(dataset);
    }

    @DeleteMapping("/id/{id}")
    public ResponseEntity<?> deleteDataset(@PathVariable("id") UUID id) {
        if (datasetService.getDatasetById(id) == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        datasetService.deleteDataset(id);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PutMapping("/id/{id}/upvote")
    public ResponseEntity<Dataset> upvote(@PathVariable("id") UUID id) {
        if (datasetService.getDatasetById(id) == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        datasetService.upvoteDataset(id);
        return new ResponseEntity<>(datasetService.getDatasetById(id), HttpStatus.OK);
    }

    @PutMapping("/id/{id}/downvote")
    public ResponseEntity<Dataset> downvote(@PathVariable("id") UUID id) {
        if (datasetService.getDatasetById(id) == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        datasetService.downvoteDataset(id);
        return new ResponseEntity<>(datasetService.getDatasetById(id), HttpStatus.OK);
    }

    @PutMapping("/id/{id}/stars")
    public ResponseEntity<Dataset> postMethodName(@PathVariable("id") UUID id,
            @RequestParam("stars") int stars) {
        if (datasetService.getDatasetById(id) == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        if (!(stars >= 0 && stars < 6)) {
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
        datasetService.voteDataset(id, stars);
        return new ResponseEntity<>(datasetService.getDatasetById(id), HttpStatus.OK);
    }

    @GetMapping("/search")
    public ResponseEntity<Page<Dataset>> search(
            @RequestParam(value = "search", required = false, defaultValue = "%") String search,
            @RequestParam(value = "page", required = false, defaultValue = "0") int page,
            @RequestParam(value = "size", required = false, defaultValue = "20") int size,
            @RequestParam(value = "sort", required = false, defaultValue = "upvotes") String sort,
            @RequestParam(value = "direction", required = false, defaultValue = "desc") String direction,
            @RequestParam(value = "category", required = false, defaultValue = "%") String category,
            @RequestParam(value = "type", required = false, defaultValue = "%") String type) {
        Pageable pageable = null;
        if (!Dataset.getSort().contains(sort))
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        try {
            pageable = PageRequest.of(page, size,
                    Sort.by(Sort.Direction.fromString(direction), sort));
        } catch (Exception e) {
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }

        return new ResponseEntity<>(datasetService.searchByOptionalCriteria(search, category, type, pageable),
                HttpStatus.OK);
    }

}