package de.uni_passau.fim.PADAS.group3.DataDash.Dataset;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;

import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;

@DataJpaTest
public class DataRepositoryTests {

    @Autowired
    private dataRepository dataRepository;
    
    @BeforeEach
    private void clearDatabase() {
        dataRepository.deleteAll();
    }

    @Test
    public void testFindByOptionalCriteria() {
        // Prepare test data
        Dataset dataset = new Dataset("title", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset);

        // Execute the method under test
        Page<Dataset> result = dataRepository.findByOptionalCriteria(
                Optional.of("title"), Optional.empty(), Optional.empty(), Optional.empty(),
                Optional.empty(), Optional.empty(), Optional.empty(), PageRequest.of(0, 10));

        // Assertions
        assertThat(result.getContent()).hasSize(1);
        assertThat(result.getContent().get(0).getTitle()).isEqualTo("title");
    }
    @Test
    public void searchByOptionalCriteria() {
        Dataset dataset1 = new Dataset("water", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset1);
        Dataset dataset = new Dataset("title1", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset);
        dataset = new Dataset("title1", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset);
         
        Page<Dataset> result = dataRepository.searchByOptionalCriteria(
                Optional.of("water"), Optional.empty(), PageRequest.of(0, 10));

            assertThat(result.getContent()).hasSize(1);
            assertThat(result.getContent().get(0).getTitle()).isEqualTo("water");
    }
    @Test
    public void searchByOptionalCriteriaMulti() {
        Dataset dataset1 = new Dataset("title", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset1);
        Dataset dataset = new Dataset("title1", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset);
        dataset = new Dataset("title1", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataRepository.save(dataset);
         
        Page<Dataset> result = dataRepository.searchByOptionalCriteria(
                Optional.of("title%"), Optional.empty(), PageRequest.of(0, 10));

            assertThat(result.getContent()).hasSize(3);
    }

    @Test
    public void getDatasetById() {
        Dataset dataset = new Dataset("title", "abst", "desc", "auth", null, null, Type.API, "MIT");
        dataset = dataRepository.save(dataset);

        Dataset result = dataRepository.getDatasetById(dataset.getId());

        assertThat(result).isNotNull();
        assertThat(result.getTitle()).isEqualTo("title");
    }   
}