package de.uni_passau.fim.PADAS.group3.DataDash.category;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;

@DataJpaTest
public class CategoryRepositoryTests {

    @Autowired
    private CategoryRepository categoryRepository;

    private Category savedCategory;

    @BeforeEach
    public void setup() {
        // Setup data for each test
        Category category = new Category();
        category.setName("Test Category");
        // Assuming Category class has a setName method and an ID that is auto-generated
        savedCategory = categoryRepository.save(category);
    }

    @Test
    public void whenFindById_thenReturnCategory() {
        Optional<Category> foundCategory = categoryRepository.findById(savedCategory.getId());
        assertThat(foundCategory).isPresent();
        assertThat(foundCategory.get().getId()).isEqualTo(savedCategory.getId());
    }

    @Test
    public void whenFindByName_thenReturnCategoryList() {
        List<Category> categories = categoryRepository.findByName("Test Category");
        assertThat(categories).hasSize(1);
        assertThat(categories.get(0).getName()).isEqualTo("Test Category");
    }

    @Test
    public void whenFindAll_thenReturnAllCategories() {
        List<Category> categories = categoryRepository.findAll();
        assertThat(categories).isNotEmpty();
    }

    @Test
    public void whenGetCategoryById_thenReturnCategory() {
        Category foundCategory = categoryRepository.getCategoryById(savedCategory.getId());
        assertThat(foundCategory).isNotNull();
        assertThat(foundCategory.getId()).isEqualTo(savedCategory.getId());
    }
}